import os
from datetime import datetime, timedelta

from django.conf import settings
from django.core.cache import cache
from django.db.models import Count
from django.http import Http404
from django.shortcuts import render, redirect

from authn.decorators.auth import require_auth
from badges.models import UserBadge
from club.exceptions import AccessDenied
from landing.forms import GodmodeNetworkSettingsEditForm, GodmodeDigestEditForm, GodmodeInviteForm
from landing.models import GodSettings
from notifications.email.invites import send_invited_email
from notifications.telegram.common import send_telegram_message, ADMIN_CHAT
from users.models.user import User

EXISTING_DOCS = [
    os.path.splitext(f)[0]  # get only filenames
    for f in os.listdir(os.path.join(settings.BASE_DIR, "frontend/html/docs"))
    if f.endswith(".html")
]


def landing(request):
    stats = cache.get("landing_stats")
    if not stats:
        stats = {
            "users": User.registered_members().count(),
            "countries": User.registered_members().values("country")
            .annotate(total=Count("country")).order_by().count() + 1,
        }
        cache.set("landing_stats", stats, settings.LANDING_CACHE_TIMEOUT)

    return render(request, "landing.html", {
        "stats": stats
    })


def docs(request, doc_slug):
    if doc_slug not in EXISTING_DOCS:
        raise Http404()

    return render(request, f"docs/{doc_slug}.html")


@require_auth
def godmode_settings(request):
    if not request.me.is_god:
        raise AccessDenied()

    return render(request, "admin/godmode.html")


@require_auth
def godmode_network_settings(request):
    if not request.me.is_god:
        raise AccessDenied()

    god_settings = GodSettings.objects.first()

    if request.method == "POST":
        form = GodmodeNetworkSettingsEditForm(request.POST, request.FILES, instance=god_settings)
        if form.is_valid():
            form.save()
            return redirect("godmode_settings")
    else:
        form = GodmodeNetworkSettingsEditForm(instance=god_settings)

    return render(request, "admin/simple_form.html", {"form": form})


@require_auth
def godmode_digest_settings(request):
    if not request.me.is_god:
        raise AccessDenied()

    god_settings = GodSettings.objects.first()

    if request.method == "POST":
        form = GodmodeDigestEditForm(request.POST, request.FILES, instance=god_settings)
        if form.is_valid():
            form.save()
            return redirect("godmode_settings")
    else:
        form = GodmodeDigestEditForm(instance=god_settings)

    return render(request, "admin/simple_form.html", {"form": form})


@require_auth
def godmode_invite(request):
    if not request.me.is_god:
        raise AccessDenied()

    if request.method == "POST":
        form = GodmodeInviteForm(request.POST, request.FILES)
        if form.is_valid():
            email = form.cleaned_data["email"]
            days = form.cleaned_data["days"]
            now = datetime.utcnow()

            user = User.objects.filter(email=email).first()
            if user:
                # add days to existing user instead of overwriting
                user.membership_expires_at = max(
                    now + timedelta(days=days),
                    user.membership_expires_at + timedelta(days=days)
                )
                user.membership_platform_type = User.MEMBERSHIP_PLATFORM_DIRECT
                user.updated_at = now
                user.save()
            else:
                # create new user with that email
                user, is_created = User.objects.get_or_create(
                    email=email,
                    defaults=dict(
                        membership_platform_type=User.MEMBERSHIP_PLATFORM_DIRECT,
                        full_name=email[:email.find("@")],
                        membership_started_at=now,
                        membership_expires_at=now + timedelta(days=days),
                        created_at=now,
                        updated_at=now,
                        moderation_status=User.MODERATION_STATUS_INTRO,
                    ),
                )

            send_invited_email(request.me, user)

            send_telegram_message(
                chat=ADMIN_CHAT,
                text=f"🎁 <b>Юзера '{email}' заинвайтили за донат</b>",
            )

            return render(request, "message.html", {
                "title": "🎁 Юзер приглашен",
                "message": f"Сейчас он получит на почту '{email}' уведомление об этом. "
                           f"Ему будет нужно залогиниться по этой почте и заполнить интро."
            })
    else:
        form = GodmodeInviteForm()

    return render(request, "admin/simple_form.html", {"form": form})


@require_auth
def badge_generator(request):
    requested_users = request.GET.get("users")
    if requested_users:
        requested_users = requested_users.split(",")
    else:
        requested_users = [request.me.slug]

    users = User.registered_members().filter(slug__in=requested_users)

    for user in users:
        user.badges = UserBadge.user_badges_grouped(user=user)

    repeat = int(request.GET.get("repeat") or 1)
    if repeat > 1:
        users = [u for u in users for _ in range(repeat)]

    # sort by name
    users = sorted(users, key=lambda u: u.full_name.lower())

    return render(request, "admin/badge_generator.html", {
        "users": users,
        "requested_users": ",".join(requested_users),
        "hide_bio": request.GET.get("hide_bio"),
        "hide_stats": request.GET.get("hide_stats"),
        "hide_badges": request.GET.get("hide_badges"),
        "repeat": repeat,
    })
